@@ -1,6 +1,8 @@
 #include "adw-demo-page-styles.h"
 
-#include "adw-style-demo-dialog.h"
+#include <glib/gi18n.h>
+
+#include "adw-style-demo-window.h"
 
 struct _AdwDemoPageStyles
 {
@@ -12,7 +14,11 @@ G_DEFINE_FINAL_TYPE (AdwDemoPageStyles, adw_demo_page_styles, ADW_TYPE_BIN)
 static void
 demo_run_cb (AdwDemoPageStyles *self)
 {
-  adw_dialog_present (adw_style_demo_dialog_new (), GTK_WIDGET (self));
+  AdwStyleDemoWindow *window = adw_style_demo_window_new ();
+  GtkRoot *root = gtk_widget_get_root (GTK_WIDGET (self));
+
+  gtk_window_set_transient_for (GTK_WINDOW (window), GTK_WINDOW (root));
+  gtk_window_present (GTK_WINDOW (window));
 }
 
 static void
