@@ -6,7 +6,7 @@
 
 /*
  * Heavily based on GTK 3.99.3 GtkDropDown widget initially written by Matthias
- * Clasen, and heavily modified for libadwaita by Alice Mikhaylenko on
+ * Clasen, and heavily modified for libadwaita by Alexander Mikhaylenko on
  * behalf of Purism SPC 2020.
  */
 
@@ -75,22 +75,16 @@ typedef struct
   GtkListView *current;
   GtkListView *list;
   GtkPopover *popover;
-  GtkSearchEntry *search_entry;
   gboolean use_subtitle;
-  gboolean enable_search;
 
-  gboolean uses_default_factory;
   GtkListItemFactory *factory;
   GtkListItemFactory *list_factory;
-  GtkListItemFactory *header_factory;
   GListModel *model;
-  GListModel *filter_model;
   GtkSelectionModel *selection;
   GtkSelectionModel *popup_selection;
   GtkSelectionModel *current_selection;
 
   GtkExpression *expression;
-  GtkStringFilterMatchMode search_match_mode;
 } AdwComboRowPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (AdwComboRow, adw_combo_row, ADW_TYPE_ACTION_ROW)
@@ -101,12 +95,9 @@ enum {
   PROP_SELECTED_ITEM,
   PROP_MODEL,
   PROP_FACTORY,
-  PROP_HEADER_FACTORY,
   PROP_LIST_FACTORY,
   PROP_EXPRESSION,
   PROP_USE_SUBTITLE,
-  PROP_ENABLE_SEARCH,
-  PROP_SEARCH_MATCH_MODE,
   LAST_PROP,
 };
 
@@ -138,48 +129,29 @@ selection_changed (AdwComboRow *self)
 {
   AdwComboRowPrivate *priv = adw_combo_row_get_instance_private (self);
   guint selected;
-  GtkFilter *filter;
 
   if (!GTK_IS_SINGLE_SELECTION (priv->selection))
     return;
 
   selected = gtk_single_selection_get_selected (GTK_SINGLE_SELECTION (priv->selection));
 
-  /* reset the filter so positions are 1-1 */
-  filter = gtk_filter_list_model_get_filter (GTK_FILTER_LIST_MODEL (priv->filter_model));
-  if (GTK_IS_STRING_FILTER (filter))
-    gtk_string_filter_set_search (GTK_STRING_FILTER (filter), "");
+  if (priv->use_subtitle) {
+    if (g_list_model_get_n_items (G_LIST_MODEL (priv->current_selection)) > 0) {
+      GtkListItem *item = g_list_model_get_item (G_LIST_MODEL (priv->current_selection), 0);
+      char *repr = get_item_representation (self, item);
+
+      adw_action_row_set_subtitle (ADW_ACTION_ROW (self), repr);
+
+      g_free (repr);
+      g_object_unref (item);
+    } else {
+      adw_action_row_set_subtitle (ADW_ACTION_ROW (self), NULL);
+    }
+  }
+
   gtk_single_selection_set_selected (GTK_SINGLE_SELECTION (priv->popup_selection), selected);
 
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_SELECTED]);
-}
-
-static void
-selection_item_changed (AdwComboRow *self)
-{
-  AdwComboRowPrivate *priv = adw_combo_row_get_instance_private (self);
-
-  if (g_list_model_get_n_items (G_LIST_MODEL (priv->current_selection)) > 0) {
-    GtkListItem *item = g_list_model_get_item (G_LIST_MODEL (priv->current_selection), 0);
-    char *repr = get_item_representation (self, item);
-
-    gtk_accessible_update_property (GTK_ACCESSIBLE (self),
-                                    GTK_ACCESSIBLE_PROPERTY_VALUE_TEXT, repr,
-                                    -1);
-
-    if (priv->use_subtitle)
-      adw_action_row_set_subtitle (ADW_ACTION_ROW (self), repr);
-
-    g_free (repr);
-    g_object_unref (item);
-  } else {
-    gtk_accessible_reset_property (GTK_ACCESSIBLE (self),
-                                   GTK_ACCESSIBLE_PROPERTY_VALUE_TEXT);
-
-    if (priv->use_subtitle)
-      adw_action_row_set_subtitle (ADW_ACTION_ROW (self), NULL);
-  }
-
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_SELECTED_ITEM]);
 }
 
@@ -189,24 +161,18 @@ model_changed (AdwComboRow *self)
   AdwComboRowPrivate *priv = adw_combo_row_get_instance_private (self);
   guint n_items = priv->model ? g_list_model_get_n_items (priv->model) : 0;
 
+  gtk_widget_set_sensitive (GTK_WIDGET (self), n_items > 0);
   gtk_widget_set_visible (priv->arrow_box, n_items > 1);
   gtk_list_box_row_set_activatable (GTK_LIST_BOX_ROW (self), n_items > 1);
 }
 
 static void
-row_activated_cb (AdwComboRow *self,
-                  guint        position)
+row_activated_cb (AdwComboRow *self)
 {
   AdwComboRowPrivate *priv = adw_combo_row_get_instance_private (self);
-  GtkFilter *filter;
 
   gtk_popover_popdown (GTK_POPOVER (priv->popover));
 
-  gtk_single_selection_set_selected (GTK_SINGLE_SELECTION (priv->popup_selection), position);
-  /* reset the filter so positions are 1-1 */
-  filter = gtk_filter_list_model_get_filter (GTK_FILTER_LIST_MODEL (priv->filter_model));
-  if (GTK_IS_STRING_FILTER (filter))
-    gtk_string_filter_set_search (GTK_STRING_FILTER (filter), "");
   adw_combo_row_set_selected (self, gtk_single_selection_get_selected (GTK_SINGLE_SELECTION (priv->popup_selection)));
 }
 
@@ -215,64 +181,10 @@ notify_popover_visible_cb (AdwComboRow *self)
 {
   AdwComboRowPrivate *priv = adw_combo_row_get_instance_private (self);
 
-  if (gtk_widget_get_visible (GTK_WIDGET (priv->popover))) {
+  if (gtk_widget_get_visible (GTK_WIDGET (priv->popover)))
     gtk_widget_add_css_class (GTK_WIDGET (self), "has-open-popup");
-  } else {
+  else
     gtk_widget_remove_css_class (GTK_WIDGET (self), "has-open-popup");
-    gtk_editable_set_text (GTK_EDITABLE (priv->search_entry), "");
-  }
-}
-
-static void
-update_filter (AdwComboRow *self)
-{
-  AdwComboRowPrivate *priv = adw_combo_row_get_instance_private (self);
-
-  if (priv->filter_model) {
-    GtkFilter *filter;
-
-    if (priv->expression) {
-      filter = GTK_FILTER (gtk_string_filter_new (gtk_expression_ref (priv->expression)));
-      gtk_string_filter_set_match_mode (GTK_STRING_FILTER (filter), priv->search_match_mode);
-    } else {
-      filter = GTK_FILTER (gtk_every_filter_new ());
-    }
-
-    gtk_filter_list_model_set_filter (GTK_FILTER_LIST_MODEL (priv->filter_model), filter);
-    g_object_unref (filter);
-  }
-}
-
-static void
-search_changed_cb (GtkSearchEntry *entry,
-                   AdwComboRow    *self)
-{
-  AdwComboRowPrivate *priv = adw_combo_row_get_instance_private (self);
-  const char *text;
-  GtkFilter *filter;
-
-  text = gtk_editable_get_text (GTK_EDITABLE (entry));
-
-  filter = gtk_filter_list_model_get_filter (GTK_FILTER_LIST_MODEL (priv->filter_model));
-  if (GTK_IS_STRING_FILTER (filter))
-    gtk_string_filter_set_search (GTK_STRING_FILTER (filter), text);
-}
-
-static void
-search_stop_cb (GtkSearchEntry *entry,
-                AdwComboRow    *self)
-{
-  AdwComboRowPrivate *priv = adw_combo_row_get_instance_private (self);
-
-  GtkFilter *filter;
-
-  filter = gtk_filter_list_model_get_filter (GTK_FILTER_LIST_MODEL (priv->filter_model));
-  if (GTK_IS_STRING_FILTER (filter)) {
-    if (gtk_string_filter_get_search (GTK_STRING_FILTER (filter)))
-      gtk_string_filter_set_search (GTK_STRING_FILTER (filter), NULL);
-    else
-      gtk_popover_popdown (GTK_POPOVER (priv->popover));
-  }
 }
 
 static void
@@ -300,14 +212,10 @@ setup_item (GtkSignalListItemFactory *factory,
   gtk_label_set_xalign (GTK_LABEL (label), 0.0);
   gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
   gtk_label_set_max_width_chars (GTK_LABEL (label), 20);
-  gtk_label_set_width_chars (GTK_LABEL (label), 1);
   gtk_widget_set_valign (label, GTK_ALIGN_CENTER);
   gtk_box_append (GTK_BOX (box), label);
 
-  icon = g_object_new (GTK_TYPE_IMAGE,
-                       "accessible-role", GTK_ACCESSIBLE_ROLE_PRESENTATION,
-                       "icon-name", "object-select-symbolic",
-                       NULL);
+  icon = gtk_image_new_from_icon_name ("object-select-symbolic");
   gtk_box_append (GTK_BOX (box), icon);
 
   gtk_list_item_set_child (list_item, box);
@@ -330,33 +238,20 @@ selected_item_changed (AdwComboRow *self,
     gtk_widget_set_opacity (icon, 0.0);
 }
 
-static void
-root_changed (GtkWidget   *box,
-              GParamSpec  *pspec,
-              AdwComboRow *self)
-{
-  AdwComboRowPrivate *priv = adw_combo_row_get_instance_private (self);
-  GtkWidget *icon;
-
-  icon = gtk_widget_get_last_child (box);
-
-  if (gtk_widget_get_ancestor (box, GTK_TYPE_POPOVER) == GTK_WIDGET (priv->popover))
-    gtk_widget_set_visible (icon, TRUE);
-  else
-    gtk_widget_set_visible (icon, FALSE);
-}
-
 static void
 bind_item (GtkSignalListItemFactory *factory,
            GtkListItem              *list_item,
            AdwComboRow              *self)
 {
+  AdwComboRowPrivate *priv = adw_combo_row_get_instance_private (self);
   gpointer item;
   GtkWidget *box;
+  GtkWidget *icon;
   char *repr;
 
   item = gtk_list_item_get_item (list_item);
   box = gtk_list_item_get_child (list_item);
+  icon = gtk_widget_get_last_child (box);
 
   repr = get_item_representation (self, item);
 
@@ -368,13 +263,14 @@ bind_item (GtkSignalListItemFactory *factory,
     g_critical ("Either AdwComboRow:factory or AdwComboRow:expression must be set");
   }
 
-  g_signal_connect (self, "notify::selected-item",
-                    G_CALLBACK (selected_item_changed), list_item);
-  selected_item_changed (self, NULL, list_item);
-
-  g_signal_connect (box, "notify::root",
-                    G_CALLBACK (root_changed), self);
-  root_changed (box, NULL, self);
+  if (gtk_widget_get_ancestor (box, GTK_TYPE_POPOVER) == GTK_WIDGET (priv->popover)) {
+    gtk_widget_set_visible (icon, TRUE);
+    g_signal_connect (self, "notify::selected-item",
+                      G_CALLBACK (selected_item_changed), list_item);
+    selected_item_changed (self, NULL, list_item);
+  } else {
+    gtk_widget_set_visible (icon, FALSE);
+  }
 
   g_clear_pointer (&repr, g_free);
 }
@@ -384,32 +280,7 @@ unbind_item (GtkSignalListItemFactory *factory,
              GtkListItem              *list_item,
              AdwComboRow              *self)
 {
-  GtkWidget *box;
-
-  box = gtk_list_item_get_child (list_item);
-
   g_signal_handlers_disconnect_by_func (self, selected_item_changed, list_item);
-  g_signal_handlers_disconnect_by_func (box, root_changed, self);
-}
-
-static void
-set_factory (AdwComboRow        *self,
-             GtkListItemFactory *factory,
-             gboolean            is_default)
-{
-  AdwComboRowPrivate *priv = adw_combo_row_get_instance_private (self);
-
-  if (!g_set_object (&priv->factory, factory))
-    return;
-
-  gtk_list_view_set_factory (priv->current, factory);
-
-  priv->uses_default_factory = is_default;
-
-  if (priv->list_factory == NULL)
-    gtk_list_view_set_factory (priv->list, factory);
-
-  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_FACTORY]);
 }
 
 static void
@@ -421,7 +292,7 @@ set_default_factory (AdwComboRow *self)
   g_signal_connect (factory, "bind", G_CALLBACK (bind_item), self);
   g_signal_connect (factory, "unbind", G_CALLBACK (unbind_item), self);
 
-  set_factory (self, factory, TRUE);
+  adw_combo_row_set_factory (self, factory);
 
   g_object_unref (factory);
 }
@@ -447,9 +318,6 @@ adw_combo_row_get_property (GObject    *object,
   case PROP_FACTORY:
     g_value_set_object (value, adw_combo_row_get_factory (self));
     break;
-  case PROP_HEADER_FACTORY:
-    g_value_set_object (value, adw_combo_row_get_header_factory (self));
-    break;
   case PROP_LIST_FACTORY:
     g_value_set_object (value, adw_combo_row_get_list_factory (self));
     break;
@@ -459,12 +327,6 @@ adw_combo_row_get_property (GObject    *object,
   case PROP_USE_SUBTITLE:
     g_value_set_boolean (value, adw_combo_row_get_use_subtitle (self));
     break;
-  case PROP_ENABLE_SEARCH:
-    g_value_set_boolean (value, adw_combo_row_get_enable_search (self));
-    break;
-  case PROP_SEARCH_MATCH_MODE:
-    g_value_set_enum (value, adw_combo_row_get_search_match_mode (self));
-    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
@@ -491,21 +353,12 @@ adw_combo_row_set_property (GObject      *object,
   case PROP_LIST_FACTORY:
     adw_combo_row_set_list_factory (self, g_value_get_object (value));
     break;
-  case PROP_HEADER_FACTORY:
-    adw_combo_row_set_header_factory (self, g_value_get_object (value));
-    break;
   case PROP_EXPRESSION:
     adw_combo_row_set_expression (self, gtk_value_get_expression (value));
     break;
   case PROP_USE_SUBTITLE:
     adw_combo_row_set_use_subtitle (self, g_value_get_boolean (value));
     break;
-  case PROP_ENABLE_SEARCH:
-    adw_combo_row_set_enable_search (self, g_value_get_boolean (value));
-    break;
-  case PROP_SEARCH_MATCH_MODE:
-    adw_combo_row_set_search_match_mode (self, g_value_get_enum (value));
-    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
@@ -522,7 +375,6 @@ adw_combo_row_dispose (GObject *object)
 
   if (priv->selection) {
     g_signal_handlers_disconnect_by_func (priv->selection, selection_changed, self);
-    g_signal_handlers_disconnect_by_func (priv->selection, selection_item_changed, self);
     g_signal_handlers_disconnect_by_func (priv->selection, model_changed, self);
   }
 
@@ -532,8 +384,6 @@ adw_combo_row_dispose (GObject *object)
   g_clear_object (&priv->current_selection);
   g_clear_object (&priv->factory);
   g_clear_object (&priv->list_factory);
-  g_clear_object (&priv->header_factory);
-  g_clear_object (&priv->filter_model);
 
   g_clear_object (&priv->model);
 
@@ -584,7 +434,7 @@ adw_combo_row_class_init (AdwComboRowClass *klass)
   row_class->activate = adw_combo_row_activate;
 
   /**
-   * AdwComboRow:selected:
+   * AdwComboRow:selected: (attributes org.gtk.Property.get=adw_combo_row_get_selected org.gtk.Property.set=adw_combo_row_set_selected)
    *
    * The position of the selected item.
    *
@@ -597,7 +447,7 @@ adw_combo_row_class_init (AdwComboRowClass *klass)
                        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwComboRow:selected-item:
+   * AdwComboRow:selected-item: (attributes org.gtk.Property.get=adw_combo_row_get_selected_item)
    *
    * The selected item.
    */
@@ -607,7 +457,7 @@ adw_combo_row_class_init (AdwComboRowClass *klass)
                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * AdwComboRow:model:
+   * AdwComboRow:model: (attributes org.gtk.Property.get=adw_combo_row_get_model org.gtk.Property.set=adw_combo_row_set_model)
    *
    * The model that provides the displayed items.
    */
@@ -617,7 +467,7 @@ adw_combo_row_class_init (AdwComboRowClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwComboRow:factory:
+   * AdwComboRow:factory: (attributes org.gtk.Property.get=adw_combo_row_get_factory org.gtk.Property.set=adw_combo_row_set_factory)
    *
    * Factory for populating list items.
    *
@@ -630,19 +480,7 @@ adw_combo_row_class_init (AdwComboRowClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwComboRow:header-factory:
-   *
-   * The factory for creating header widgets for the popup.
-   *
-   * Since: 1.6
-   */
-  props[PROP_HEADER_FACTORY] =
-    g_param_spec_object ("header-factory", NULL, NULL,
-                         GTK_TYPE_LIST_ITEM_FACTORY,
-                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
-
-  /**
-   * AdwComboRow:list-factory:
+   * AdwComboRow:list-factory: (attributes org.gtk.Property.get=adw_combo_row_get_list_factory org.gtk.Property.set=adw_combo_row_set_list_factory)
    *
    * The factory for populating list items in the popup.
    *
@@ -654,7 +492,7 @@ adw_combo_row_class_init (AdwComboRowClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwComboRow:expression: (type GtkExpression)
+   * AdwComboRow:expression: (type GtkExpression) (attributes org.gtk.Property.get=adw_combo_row_get_expression org.gtk.Property.set=adw_combo_row_set_expression)
    *
    * An expression used to obtain strings from items.
    *
@@ -671,7 +509,7 @@ adw_combo_row_class_init (AdwComboRowClass *klass)
                                G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwComboRow:use-subtitle:
+   * AdwComboRow:use-subtitle: (attributes org.gtk.Property.get=adw_combo_row_get_use_subtitle org.gtk.Property.set=adw_combo_row_set_use_subtitle)
    *
    * Whether to use the current value as the subtitle.
    *
@@ -688,36 +526,6 @@ adw_combo_row_class_init (AdwComboRowClass *klass)
                           FALSE,
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
-  /**
-   * AdwComboRow:enable-search:
-   *
-   * Whether to show a search entry in the popup.
-   *
-   * If set to `TRUE`, a search entry will be shown in the popup that
-   * allows to search for items in the list.
-   * 
-   * Search requires [property@ComboRow:expression] to be set.
-   * 
-   * Since: 1.4
-   */
-  props[PROP_ENABLE_SEARCH] =
-    g_param_spec_boolean ("enable-search", NULL, NULL,
-                          FALSE,
-                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
-
-  /**
-   * AdwComboRow:search-match-mode:
-   *
-   * The match mode for the search filter.
-   *
-   * Since: 1.6
-   */
-  props[PROP_SEARCH_MATCH_MODE] =
-    g_param_spec_enum  ("search-match-mode", NULL, NULL,
-                        GTK_TYPE_STRING_FILTER_MATCH_MODE,
-                        GTK_STRING_FILTER_MATCH_MODE_PREFIX,
-                        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
-
   g_object_class_install_properties (object_class, LAST_PROP, props);
 
   gtk_widget_class_set_template_from_resource (widget_class,
@@ -726,11 +534,8 @@ adw_combo_row_class_init (AdwComboRowClass *klass)
   gtk_widget_class_bind_template_child_private (widget_class, AdwComboRow, arrow_box);
   gtk_widget_class_bind_template_child_private (widget_class, AdwComboRow, list);
   gtk_widget_class_bind_template_child_private (widget_class, AdwComboRow, popover);
-  gtk_widget_class_bind_template_child_private (widget_class, AdwComboRow, search_entry);
   gtk_widget_class_bind_template_callback (widget_class, row_activated_cb);
   gtk_widget_class_bind_template_callback (widget_class, notify_popover_visible_cb);
-  gtk_widget_class_bind_template_callback (widget_class, search_changed_cb);
-  gtk_widget_class_bind_template_callback (widget_class, search_stop_cb);
 
   gtk_widget_class_set_accessible_role (widget_class, GTK_ACCESSIBLE_ROLE_COMBO_BOX);
 }
@@ -738,10 +543,6 @@ adw_combo_row_class_init (AdwComboRowClass *klass)
 static void
 adw_combo_row_init (AdwComboRow *self)
 {
-  AdwComboRowPrivate *priv = adw_combo_row_get_instance_private (self);
-
-  priv->search_match_mode = GTK_STRING_FILTER_MATCH_MODE_PREFIX;
-
   gtk_widget_init_template (GTK_WIDGET (self));
 
   adw_preferences_row_set_use_markup (ADW_PREFERENCES_ROW (self), FALSE);
@@ -763,7 +564,7 @@ adw_combo_row_new (void)
 }
 
 /**
- * adw_combo_row_set_selected:
+ * adw_combo_row_set_selected: (attributes org.gtk.Method.set_property=selected)
  * @self: a combo row
  * @position: the position of the item to select, or
  *   [const@Gtk.INVALID_LIST_POSITION]
@@ -790,7 +591,7 @@ adw_combo_row_set_selected (AdwComboRow *self,
 }
 
 /**
- * adw_combo_row_get_selected:
+ * adw_combo_row_get_selected: (attributes org.gtk.Method.get_property=selected)
  * @self: a combo row
  *
  * Gets the position of the selected item.
@@ -814,7 +615,7 @@ adw_combo_row_get_selected (AdwComboRow *self)
 }
 
 /**
- * adw_combo_row_get_selected_item:
+ * adw_combo_row_get_selected_item: (attributes org.gtk.Method.get_property=selected-item)
  * @self: a combo row
  *
  * Gets the selected item.
@@ -837,7 +638,7 @@ adw_combo_row_get_selected_item (AdwComboRow *self)
 }
 
 /**
- * adw_combo_row_get_model:
+ * adw_combo_row_get_model: (attributes org.gtk.Method.get_property=model)
  * @self: a combo row
  *
  * Gets the model that provides the displayed items.
@@ -857,7 +658,7 @@ adw_combo_row_get_model (AdwComboRow *self)
 }
 
 /**
- * adw_combo_row_set_model:
+ * adw_combo_row_set_model: (attributes org.gtk.Method.set_property=model)
  * @self: a combo row
  * @model: (nullable) (transfer none): the model to use
  *
@@ -883,7 +684,6 @@ adw_combo_row_set_model (AdwComboRow *self,
 
     if (priv->selection) {
       g_signal_handlers_disconnect_by_func (priv->selection, selection_changed, self);
-      g_signal_handlers_disconnect_by_func (priv->selection, selection_item_changed, self);
       g_signal_handlers_disconnect_by_func (priv->selection, model_changed, self);
     }
 
@@ -892,15 +692,9 @@ adw_combo_row_set_model (AdwComboRow *self,
     g_clear_object (&priv->current_selection);
   } else {
     GtkSelectionModel *selection;
-    GListModel *filter_model;
     GListModel *current_model;
 
-    filter_model = G_LIST_MODEL (gtk_filter_list_model_new (g_object_ref (model), NULL));
-    g_set_object (&priv->filter_model, filter_model);
-
-    update_filter (self);
-
-    selection = GTK_SELECTION_MODEL (gtk_single_selection_new (g_object_ref (filter_model)));
+    selection = GTK_SELECTION_MODEL (gtk_single_selection_new (g_object_ref (model)));
     g_set_object (&priv->popup_selection, selection);
     gtk_list_view_set_model (priv->list, selection);
     g_object_unref (selection);
@@ -909,8 +703,6 @@ adw_combo_row_set_model (AdwComboRow *self,
     g_set_object (&priv->selection, selection);
     g_object_unref (selection);
 
-    g_object_unref (filter_model);
-
     current_model = G_LIST_MODEL (gtk_selection_filter_model_new (priv->selection));
     selection = GTK_SELECTION_MODEL (gtk_no_selection_new (current_model));
     g_set_object (&priv->current_selection, selection);
@@ -918,11 +710,9 @@ adw_combo_row_set_model (AdwComboRow *self,
     g_object_unref (selection);
 
     g_signal_connect_swapped (priv->selection, "notify::selected", G_CALLBACK (selection_changed), self);
-    g_signal_connect_swapped (priv->selection, "notify::selected-item", G_CALLBACK (selection_item_changed), self);
     g_signal_connect_swapped (priv->selection, "items-changed", G_CALLBACK (model_changed), self);
 
     selection_changed (self);
-    selection_item_changed (self);
     model_changed (self);
   }
 
@@ -930,7 +720,7 @@ adw_combo_row_set_model (AdwComboRow *self,
 }
 
 /**
- * adw_combo_row_get_factory:
+ * adw_combo_row_get_factory: (attributes org.gtk.Method.get_property=factory)
  * @self: a combo row
  *
  * Gets the factory for populating list items.
@@ -950,7 +740,7 @@ adw_combo_row_get_factory (AdwComboRow *self)
 }
 
 /**
- * adw_combo_row_set_factory:
+ * adw_combo_row_set_factory: (attributes org.gtk.Method.set_property=factory)
  * @self: a combo row
  * @factory: (nullable) (transfer none): the factory to use
  *
@@ -962,47 +752,6 @@ adw_combo_row_get_factory (AdwComboRow *self)
 void
 adw_combo_row_set_factory (AdwComboRow        *self,
                            GtkListItemFactory *factory)
-{
-  g_return_if_fail (ADW_IS_COMBO_ROW (self));
-  g_return_if_fail (factory == NULL || GTK_LIST_ITEM_FACTORY (factory));
-
-  set_factory (self, factory, FALSE);
-}
-
-/**
- * adw_combo_row_get_header_factory:
- * @self: a combo row
- *
- * Gets the factory that's currently used to create header widgets for the popup.
- *
- * Returns: (nullable) (transfer none): The factory in use
- *
- * Since: 1.6
- */
-GtkListItemFactory *
-adw_combo_row_get_header_factory (AdwComboRow *self)
-{
-  AdwComboRowPrivate *priv;
-
-  g_return_val_if_fail (ADW_IS_COMBO_ROW (self), NULL);
-
-  priv = adw_combo_row_get_instance_private (self);
-
-  return priv->header_factory;
-}
-
-/**
- * adw_combo_row_set_header_factory:
- * @self: a combo row
- * @factory: (nullable) (transfer none): the factory to use
- *
- * Sets the factory to use for creating header widgets for the popup.
- *
- * Since: 1.6
- */
-void
-adw_combo_row_set_header_factory (AdwComboRow        *self,
-                                  GtkListItemFactory *factory)
 {
   AdwComboRowPrivate *priv;
 
@@ -1011,16 +760,19 @@ adw_combo_row_set_header_factory (AdwComboRow        *self,
 
   priv = adw_combo_row_get_instance_private (self);
 
-  if (!g_set_object (&priv->header_factory, factory))
+  if (!g_set_object (&priv->factory, factory))
     return;
 
-  gtk_list_view_set_header_factory (GTK_LIST_VIEW (priv->list), priv->header_factory);
+  gtk_list_view_set_factory (priv->current, factory);
 
-  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_HEADER_FACTORY]);
+  if (priv->list_factory == NULL)
+    gtk_list_view_set_factory (priv->list, factory);
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_FACTORY]);
 }
 
 /**
- * adw_combo_row_get_list_factory:
+ * adw_combo_row_get_list_factory: (attributes org.gtk.Method.get_property=list-factory)
  * @self: a combo row
  *
  * Gets the factory for populating list items in the popup.
@@ -1040,7 +792,7 @@ adw_combo_row_get_list_factory (AdwComboRow *self)
 }
 
 /**
- * adw_combo_row_set_list_factory:
+ * adw_combo_row_set_list_factory: (attributes org.gtk.Method.set_property=list-factory)
  * @self: a combo row
  * @factory: (nullable) (transfer none): the factory to use
  *
@@ -1071,7 +823,7 @@ adw_combo_row_set_list_factory (AdwComboRow        *self,
 }
 
 /**
- * adw_combo_row_get_expression:
+ * adw_combo_row_get_expression: (attributes org.gtk.Method.get_property=expression)
  * @self: a combo row
  *
  * Gets the expression used to obtain strings from items.
@@ -1091,7 +843,7 @@ adw_combo_row_get_expression (AdwComboRow *self)
 }
 
 /**
- * adw_combo_row_set_expression:
+ * adw_combo_row_set_expression: (attributes org.gtk.Method.set_property=expression)
  * @self: a combo row
  * @expression: (nullable): an expression
  *
@@ -1126,16 +878,11 @@ adw_combo_row_set_expression (AdwComboRow   *self,
 
   selection_changed (self);
 
-  update_filter (self);
-
-  if (priv->uses_default_factory)
-    set_default_factory (self);
-
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_EXPRESSION]);
 }
 
 /**
- * adw_combo_row_get_use_subtitle:
+ * adw_combo_row_get_use_subtitle: (attributes org.gtk.Method.get_property=use-subtitle)
  * @self: a combo row
  *
  * Gets whether to use the current value as the subtitle.
@@ -1155,7 +902,7 @@ adw_combo_row_get_use_subtitle (AdwComboRow *self)
 }
 
 /**
- * adw_combo_row_set_use_subtitle:
+ * adw_combo_row_set_use_subtitle: (attributes org.gtk.Method.set_property=use-subtitle)
  * @self: a combo row
  * @use_subtitle: whether to use the current value as the subtitle
  *
@@ -1191,118 +938,3 @@ adw_combo_row_set_use_subtitle (AdwComboRow *self,
 
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_USE_SUBTITLE]);
 }
-
-/**
- * adw_combo_row_get_enable_search:
- * @self: a combo row
- *
- * Gets whether search is enabled.
- * 
- * If set to `TRUE`, a search entry will be shown in the popup that
- * allows to search for items in the list.
- * 
- * Search requires [property@ComboRow:expression] to be set.
- *
- * Returns: whether the popup includes a search entry
- * 
- * Since: 1.4
- */
-gboolean
-adw_combo_row_get_enable_search (AdwComboRow *self)
-{
-  AdwComboRowPrivate *priv;
-
-  g_return_val_if_fail (ADW_IS_COMBO_ROW (self), FALSE);
-
-  priv = adw_combo_row_get_instance_private (self);
-
-  return priv->enable_search;
-}
-
-/**
- * adw_combo_row_set_enable_search:
- * @self: a combo row
- * @enable_search: whether to enable search
- *
- * Sets whether to enable search.
- * 
- * If set to `TRUE`, a search entry will be shown in the popup that
- * allows to search for items in the list.
- * 
- * Search requires [property@ComboRow:expression] to be set.
- * 
- * Since: 1.4
- */
-void
-adw_combo_row_set_enable_search (AdwComboRow *self,
-                                 gboolean     enable_search)
-{
-  AdwComboRowPrivate *priv;
-
-  g_return_if_fail (ADW_IS_COMBO_ROW (self));
-
-  priv = adw_combo_row_get_instance_private (self);
-
-  enable_search = !!enable_search;
-
-  if (priv->enable_search == enable_search)
-    return;
-
-  priv->enable_search = enable_search;
-  
-  gtk_editable_set_text (GTK_EDITABLE (priv->search_entry), "");
-  gtk_widget_set_visible (GTK_WIDGET (priv->search_entry), enable_search);
-
-  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_ENABLE_SEARCH]);
-}
-
-/**
- * adw_combo_row_get_search_match_mode:
- * @self: a combo row
- *
- * Returns the match mode that the search filter is using.
- *
- * Returns: the match mode of the search filter
- *
- * Since: 1.6
- */
-GtkStringFilterMatchMode
-adw_combo_row_get_search_match_mode (AdwComboRow *self)
-{
-  AdwComboRowPrivate *priv;
-
-  g_return_val_if_fail (ADW_IS_COMBO_ROW (self), GTK_STRING_FILTER_MATCH_MODE_PREFIX);
-
-  priv = adw_combo_row_get_instance_private (self);
-
-  return priv->search_match_mode;
-}
-
-/**
- * adw_combo_row_set_search_match_mode:
- * @self: a combo row
- * @search_match_mode: the new match mode
- *
- * Sets the match mode for the search filter.
- *
- * Since: 1.6
- */
-void
-adw_combo_row_set_search_match_mode (AdwComboRow *self,
-                                     GtkStringFilterMatchMode search_match_mode)
-{
-  AdwComboRowPrivate *priv;
-
-  g_return_if_fail (ADW_IS_COMBO_ROW (self));
-
-  priv = adw_combo_row_get_instance_private (self);
-
-  if (priv->search_match_mode == search_match_mode)
-    return;
-
-  priv->search_match_mode = search_match_mode;
-
-  update_filter (self);
-
-  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_SEARCH_MATCH_MODE]);
-}
