@@ -983,7 +983,7 @@ adw_flap_measure (GtkWidget      *widget,
     }
 
     min = MAX (content_min + (int) round ((flap_min + separator_min) * min_progress), flap_min);
-    nat = MAX (content_nat + (int) round ((flap_nat + separator_min) * nat_progress), flap_nat);
+    nat = MAX (content_nat + (int) round ((flap_nat + separator_nat) * nat_progress), flap_nat);
   } else {
     min = MAX (MAX (content_min, flap_min), separator_min);
     nat = MAX (MAX (content_nat, flap_nat), separator_nat);
@@ -1225,10 +1225,9 @@ adw_flap_dispose (GObject *object)
 {
   AdwFlap *self = ADW_FLAP (object);
 
-  adw_flap_set_flap (self, NULL);
-  adw_flap_set_separator (self, NULL);
-  adw_flap_set_content (self, NULL);
-
+  g_clear_pointer (&self->flap.widget, gtk_widget_unparent);
+  g_clear_pointer (&self->separator.widget, gtk_widget_unparent);
+  g_clear_pointer (&self->content.widget, gtk_widget_unparent);
   g_clear_pointer (&self->shield, gtk_widget_unparent);
 
   g_clear_object (&self->shadow_helper);
@@ -1351,7 +1350,7 @@ adw_flap_class_init (AdwFlapClass *klass)
                           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * AdwFlap:fold-policy: (attributes org.gtk.Property.get=adw_flap_get_fold_policy)
+   * AdwFlap:fold-policy: (attributes org.gtk.Property.get=adw_flap_get_fold_policy org.gtk.Property.set=adw_flap_set_fold_policy)
    *
    * The fold policy for the flap.
    */
