@@ -8,7 +8,6 @@
 
 #include "adw-preferences-page-private.h"
 
-#include "adw-macros-private.h"
 #include "adw-preferences-group-private.h"
 #include "adw-widget-utils-private.h"
 
@@ -32,8 +31,6 @@
  * ## Accessibility
  *
  * `AdwPreferencesPage` uses the `GTK_ACCESSIBLE_ROLE_GROUP` role.
- *
- * Since: 1.0
  */
 
 typedef struct
@@ -124,10 +121,7 @@ adw_preferences_page_set_property (GObject      *object,
 static void
 adw_preferences_page_dispose (GObject *object)
 {
-  AdwPreferencesPage *self = ADW_PREFERENCES_PAGE (object);
-  AdwPreferencesPagePrivate *priv = adw_preferences_page_get_instance_private (self);
-
-  gtk_widget_unparent (priv->scrolled_window);
+  gtk_widget_dispose_template (GTK_WIDGET (object), ADW_TYPE_PREFERENCES_PAGE);
 
   G_OBJECT_CLASS (adw_preferences_page_parent_class)->dispose (object);
 }
@@ -162,8 +156,6 @@ adw_preferences_page_class_init (AdwPreferencesPageClass *klass)
    * AdwPreferencesPage:icon-name: (attributes org.gtk.Property.get=adw_preferences_page_get_icon_name org.gtk.Property.set=adw_preferences_page_set_icon_name)
    *
    * The icon name for this page.
-   *
-   * Since: 1.0
    */
   props[PROP_ICON_NAME] =
     g_param_spec_string ("icon-name", NULL, NULL,
@@ -174,8 +166,6 @@ adw_preferences_page_class_init (AdwPreferencesPageClass *klass)
    * AdwPreferencesPage:title: (attributes org.gtk.Property.get=adw_preferences_page_get_title org.gtk.Property.set=adw_preferences_page_set_title)
    *
    * The title for this page.
-   *
-   * Since: 1.0
    */
   props[PROP_TITLE] =
     g_param_spec_string ("title", NULL, NULL,
@@ -186,8 +176,6 @@ adw_preferences_page_class_init (AdwPreferencesPageClass *klass)
    * AdwPreferencesPage:name: (attributes org.gtk.Property.get=adw_preferences_page_get_name org.gtk.Property.set=adw_preferences_page_set_name)
    *
    * The name of this page.
-   *
-   * Since: 1.0
    */
   props[PROP_NAME] =
     g_param_spec_string ("name", NULL, NULL,
@@ -198,8 +186,6 @@ adw_preferences_page_class_init (AdwPreferencesPageClass *klass)
    * AdwPreferencesPage:use-underline: (attributes org.gtk.Property.get=adw_preferences_page_get_use_underline org.gtk.Property.set=adw_preferences_page_set_use_underline)
    *
    * Whether an embedded underline in the title indicates a mnemonic.
-   *
-   * Since: 1.0
    */
   props[PROP_USE_UNDERLINE] =
     g_param_spec_boolean ("use-underline", NULL, NULL,
@@ -255,8 +241,6 @@ adw_preferences_page_buildable_init (GtkBuildableIface *iface)
  * Creates a new `AdwPreferencesPage`.
  *
  * Returns: the newly created `AdwPreferencesPage`
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_preferences_page_new (void)
@@ -270,8 +254,6 @@ adw_preferences_page_new (void)
  * @group: the group to add
  *
  * Adds a preferences group to @self.
- *
- * Since: 1.0
  */
 void
 adw_preferences_page_add (AdwPreferencesPage  *self,
@@ -293,8 +275,6 @@ adw_preferences_page_add (AdwPreferencesPage  *self,
  * @group: the group to remove
  *
  * Removes a group from @self.
- *
- * Since: 1.0
  */
 void
 adw_preferences_page_remove (AdwPreferencesPage  *self,
@@ -320,8 +300,6 @@ adw_preferences_page_remove (AdwPreferencesPage  *self,
  * Gets the icon name for @self.
  *
  * Returns: (nullable): the icon name for @self
- *
- * Since: 1.0
  */
 const char *
 adw_preferences_page_get_icon_name (AdwPreferencesPage *self)
@@ -341,8 +319,6 @@ adw_preferences_page_get_icon_name (AdwPreferencesPage *self)
  * @icon_name: (nullable): the icon name
  *
  * Sets the icon name for @self.
- *
- * Since: 1.0
  */
 void
 adw_preferences_page_set_icon_name (AdwPreferencesPage *self,
@@ -370,8 +346,6 @@ adw_preferences_page_set_icon_name (AdwPreferencesPage *self,
  * Gets the title of @self.
  *
  * Returns: the title of @self.
- *
- * Since: 1.0
  */
 const char *
 adw_preferences_page_get_title (AdwPreferencesPage *self)
@@ -391,8 +365,6 @@ adw_preferences_page_get_title (AdwPreferencesPage *self)
  * @title: the title
  *
  * Sets the title of @self.
- *
- * Since: 1.0
  */
 void
 adw_preferences_page_set_title (AdwPreferencesPage *self,
@@ -420,8 +392,6 @@ adw_preferences_page_set_title (AdwPreferencesPage *self,
  * Gets the name of @self.
  *
  * Returns: (nullable): the name of @self
- *
- * Since: 1.0
  */
 const char *
 adw_preferences_page_get_name (AdwPreferencesPage *self)
@@ -441,8 +411,6 @@ adw_preferences_page_get_name (AdwPreferencesPage *self)
  * @name: (nullable): the name
  *
  * Sets the name of @self.
- *
- * Since: 1.0
  */
 void
 adw_preferences_page_set_name (AdwPreferencesPage *self,
@@ -470,8 +438,6 @@ adw_preferences_page_set_name (AdwPreferencesPage *self,
  * Gets whether an embedded underline in the title indicates a mnemonic.
  *
  * Returns: whether an embedded underline in the title indicates a mnemonic
- *
- * Since: 1.0
  */
 gboolean
 adw_preferences_page_get_use_underline (AdwPreferencesPage *self)
@@ -491,8 +457,6 @@ adw_preferences_page_get_use_underline (AdwPreferencesPage *self)
  * @use_underline: `TRUE` if underlines in the text indicate mnemonics
  *
  * Sets whether an embedded underline in the title indicates a mnemonic.
- *
- * Since: 1.0
  */
 void
 adw_preferences_page_set_use_underline (AdwPreferencesPage *self,
@@ -531,8 +495,6 @@ preferences_group_to_rows (AdwPreferencesGroup *group)
  * This can be used to keep an up-to-date view.
  *
  * Returns: (transfer full): a list model for the page's rows
- *
- * Since: 1.0
  */
 GListModel *
 adw_preferences_page_get_rows (AdwPreferencesPage *self)
@@ -556,3 +518,25 @@ adw_preferences_page_get_rows (AdwPreferencesPage *self)
 
   return G_LIST_MODEL (gtk_flatten_list_model_new (model));
 }
+
+/**
+ * adw_preferences_page_scroll_to_top:
+ * @self: a preferences page
+ *
+ * Scrolls the scrolled window of @self to the top.
+ *
+ * Since: 1.3
+ */
+void
+adw_preferences_page_scroll_to_top (AdwPreferencesPage *self)
+{
+  AdwPreferencesPagePrivate *priv;
+  GtkAdjustment *adjustment;
+
+  g_return_if_fail (ADW_IS_PREFERENCES_PAGE (self));
+
+  priv = adw_preferences_page_get_instance_private (self);
+  adjustment = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (priv->scrolled_window));
+
+  gtk_adjustment_set_value (adjustment, gtk_adjustment_get_lower (adjustment));
+}
