@@ -8,7 +8,6 @@
 #include "config.h"
 
 #include "adw-enums.h"
-#include "adw-macros-private.h"
 #include "adw-view-switcher-bar.h"
 
 /**
@@ -68,8 +67,6 @@
  * ## CSS nodes
  *
  * `AdwViewSwitcherBar` has a single CSS node with name` viewswitcherbar`.
- *
- * Since: 1.0
  */
 
 enum {
@@ -171,8 +168,7 @@ adw_view_switcher_bar_dispose (GObject *object)
     g_clear_object (&self->pages);
   }
 
-  gtk_widget_unparent (self->action_bar);
-  self->view_switcher = NULL;
+  gtk_widget_dispose_template (GTK_WIDGET (self), ADW_TYPE_VIEW_SWITCHER_BAR);
 
   G_OBJECT_CLASS (adw_view_switcher_bar_parent_class)->dispose (object);
 }
@@ -191,8 +187,6 @@ adw_view_switcher_bar_class_init (AdwViewSwitcherBarClass *klass)
    * AdwViewSwitcherBar:stack: (attributes org.gtk.Property.get=adw_view_switcher_bar_get_stack org.gtk.Property.set=adw_view_switcher_bar_set_stack)
    *
    * The stack the view switcher controls.
-   *
-   * Since: 1.0
    */
   props[PROP_STACK] =
     g_param_spec_object ("stack", NULL, NULL,
@@ -203,8 +197,6 @@ adw_view_switcher_bar_class_init (AdwViewSwitcherBarClass *klass)
    * AdwViewSwitcherBar:reveal: (attributes org.gtk.Property.get=adw_view_switcher_bar_get_reveal org.gtk.Property.set=adw_view_switcher_bar_set_reveal)
    *
    * Whether the bar should be revealed or hidden.
-   *
-   * Since: 1.0
    */
   props[PROP_REVEAL] =
     g_param_spec_boolean ("reveal", NULL, NULL,
@@ -236,8 +228,6 @@ adw_view_switcher_bar_init (AdwViewSwitcherBar *self)
  * Creates a new `AdwViewSwitcherBar`.
  *
  * Returns: the newly created `AdwViewSwitcherBar`
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_view_switcher_bar_new (void)
@@ -252,8 +242,6 @@ adw_view_switcher_bar_new (void)
  * Gets the stack controlled by @self.
  *
  * Returns: (nullable) (transfer none): the stack
- *
- * Since: 1.0
  */
 AdwViewStack *
 adw_view_switcher_bar_get_stack (AdwViewSwitcherBar *self)
@@ -269,8 +257,6 @@ adw_view_switcher_bar_get_stack (AdwViewSwitcherBar *self)
  * @stack: (nullable): a stack
  *
  * Sets the stack controlled by @self.
- *
- * Since: 1.0
  */
 void
 adw_view_switcher_bar_set_stack (AdwViewSwitcherBar *self,
@@ -311,8 +297,6 @@ adw_view_switcher_bar_set_stack (AdwViewSwitcherBar *self,
  * Gets whether @self should be revealed or hidden.
  *
  * Returns: whether @self is revealed
- *
- * Since: 1.0
  */
 gboolean
 adw_view_switcher_bar_get_reveal (AdwViewSwitcherBar *self)
@@ -328,8 +312,6 @@ adw_view_switcher_bar_get_reveal (AdwViewSwitcherBar *self)
  * @reveal: whether to reveal @self
  *
  * Sets whether @self should be revealed or hidden.
- *
- * Since: 1.0
  */
 void
 adw_view_switcher_bar_set_reveal (AdwViewSwitcherBar *self,
