@@ -3,15 +3,17 @@
  *
  * SPDX-License-Identifier: LGPL-2.1-or-later
  *
- * Author: Alice Mikhaylenko <alice.mikhaylenko@puri.sm>
+ * Author: Alexander Mikhaylenko <alexander.mikhaylenko@puri.sm>
  */
 
 #include <adwaita.h>
 
+int notified;
+
 static void
-increment (int *data)
+notify_cb (GtkWidget *widget, gpointer data)
 {
-  (*data)++;
+  notified++;
 }
 
 static void
@@ -19,11 +21,11 @@ test_adw_button_content_icon_name (void)
 {
   AdwButtonContent *content = g_object_ref_sink (ADW_BUTTON_CONTENT (adw_button_content_new ()));
   char *icon_name;
-  int notified = 0;
 
   g_assert_nonnull (content);
 
-  g_signal_connect_swapped (content, "notify::icon-name", G_CALLBACK (increment), &notified);
+  notified = 0;
+  g_signal_connect (content, "notify::icon-name", G_CALLBACK (notify_cb), NULL);
 
   g_object_get (content, "icon-name", &icon_name, NULL);
   g_assert_cmpstr (icon_name, ==, "");
@@ -48,11 +50,11 @@ test_adw_button_content_label (void)
 {
   AdwButtonContent *content = g_object_ref_sink (ADW_BUTTON_CONTENT (adw_button_content_new ()));
   char *label;
-  int notified = 0;
 
   g_assert_nonnull (content);
 
-  g_signal_connect_swapped (content, "notify::label", G_CALLBACK (increment), &notified);
+  notified = 0;
+  g_signal_connect (content, "notify::label", G_CALLBACK (notify_cb), NULL);
 
   g_object_get (content, "label", &label, NULL);
   g_assert_cmpstr (label, ==, "");
@@ -77,11 +79,11 @@ test_adw_button_content_use_underline (void)
 {
   AdwButtonContent *content = g_object_ref_sink (ADW_BUTTON_CONTENT (adw_button_content_new ()));
   gboolean use_underline;
-  int notified = 0;
 
   g_assert_nonnull (content);
 
-  g_signal_connect_swapped (content, "notify::use-underline", G_CALLBACK (increment), &notified);
+  notified = 0;
+  g_signal_connect (content, "notify::use-underline", G_CALLBACK (notify_cb), NULL);
 
   g_object_get (content, "use-underline", &use_underline, NULL);
   g_assert_false (use_underline);
